// p_subs2a_2011.c

#include "binloud.h"

/*
double mid_corr(double freqHz)
double diff_cor(double freqHz)
double Shaw_cor(double freqHz)

ex_table(double frqHz);
alpha100(double G_dB);									NOT USED in l2003
double Btable(double alpha);
double calc_K(double freqkHz);					used in binloud for TEN noise
double so2phons(double sones);
double lin_int(double *yy,double *xx, double x, unsigned long n)
void locate(double *xx, long n, double x, long *j)
double fqtoerb (double fq)					NOT USED in l2003
double erbtofq(double E)
int power(int x,int n)					NOT USED in l2003
void scalefft(double *rwave, long npts, double dBmax, long ndata)
void av_sones(double sones, double frame_ms,double *max1, double *max2,
double *min1, double *min2, short *first_time, long block,long av_from, short sig_end)
*/


/*-------------------------------------------------------------*/
double mid_corr(double freqHz)
{
	/*
		 difference between elc correction and ratio of eardrum pressure to
		 * free-field pressure given by Shaw E.A.G. Transformation of sound
		 * pressure level from the free field to the eardrum in the horizontal plane
		 * JASA 56 (1974) 1848-1861
		 * The values used were obtained from our elc correction and Tables I col B
		 * Killion M.C. Revised estimate of minimal audible pressure: Where is
		 * the 'missing 6 dB'? JASA 63 (1978) 1501-1510
		 */

#define NMID 41

  double Hz[NMID] = {
		   0.,
		  20.,     25.,   31.5,   40.,   50.,   63.,    80.,		100.,	
		 125.,    160.,   200.,  250.,  315.,  400.,   500., 		630.,
		 750.,    800.,  1000., 1250., 1500., 1600.,  2000.,   2500.,
		 3000. , 3150.,  4000., 5000., 6000., 6300.,  8000.,   9000.,
		 10000.,11200., 12500.,14000.,15000.,16000., 18000.,  20000
	};


  // Feb 2006 to get Kurakata et al abs thrs   Acoust Sci & Tech.26.5 (2005) p447
	double midear[NMID] = {
		  50.0,
      39.6,   32.0,  25.85,  21.4,   18.5,  15.9,  14.1,  12.4,
      11.0,    9.6,   8.3,    7.4,    6.2,   4.8,   3.8,   3.3,
       2.9,    2.6,   2.6,    4.5,    5.4,   6.1,   8.5,  10.4,
       7.3,    7.0,   6.6,    7.0,    9.2,  10.2,  12.2,  10.8,
      10.1,   12.7,  15.0,   18.2,   23.8,  32.3,  46.5,  50.0
	};
  return(lin_int(midear-1,Hz-1,freqHz,NMID));
}

/*-------------------------------------------------------------*/
double diff_cor(double freqHz)// Correction for diffuse field
{

#define NDIFF 40

	double Hz[NDIFF] = {0, 
	     20,    25,  31.5,    40,    50,    63,    80,  100,
		  125,   160,   200,   250,   315,   400,   500,  630, 
		  750,   800,  1000,  1250,  1500,  1600,  2000, 2500, 
		 3000,  3150,  4000,  5000,  6000,  6300,  8000, 9000, 
		10000, 11200, 12500, 14000, 15000, 16000, 20000
	};
	double diffuse[NDIFF] = {0,
      0.0,    0.0,  0.0,   0.0,   0.0,   0.0,   0.0,   0.0,
      0.1,    0.3,  0.4,   0.5,   1.0,   1.6,   1.7,   2.2,
      2.7,    2.9,  3.8,   5.3,   6.8,   7.2,  10.2,  14.9,
     14.5,   14.4, 12.7,  10.8,   8.9,   8.7,   8.5,   6.2,
      5.0,    4.5,  4.0,   3.3,   2.6,   2.0,   2.0
	};
		
	return(lin_int(diffuse-1,Hz-1,freqHz,NDIFF));
}
/*-------------------------------------------------------------*/

double Shaw_cor(double freqHz)   

	/*
		 difference between elc correction and ratio of eardrum pressure to
		 * free-field pressure given by Shaw E.A.G. Transformation of sound
		 * pressure level from the free field to the eardrum in the horizontal plane
		 * JASA 56 (1974) 1848-1861
		 * The values used were obtained from our elc correction and Tables I col B
		 * Killion M.C. Revised estimate of minimal audible pressure: Where is
		 * the 'missing 6 dB'? JASA 63 (1978) 1501-1510
		 */
{

#define NSHAW 40

  double Hz[NSHAW] = {0, 
	     20,    25,  31.5,    40,    50,    63,    80,  100,
		  125,   160,   200,   250,   315,   400,   500,  630, 
		  750,   800,  1000,  1250,  1500,  1600,  2000, 2500, 
		 3000,  3150,  4000,  5000,  6000,  6300,  8000, 9000, 
		10000, 11200, 12500, 14000, 15000, 16000, 20000
	};

	double ff_ed[NSHAW] = {0.0, 
		  0.0,   0.0,   0.0,   0.0,   0.0,   0.0,   0.0,  0.0, 
		  0.1,   0.3,   0.5,   0.9,   1.4,   1.6,   1.7,  2.5, 
		  2.7,   2.6,   2.6,	 3.2,   5.2,   6.6,  12.0, 16.8, 
		 15.3,  15.2,  14.2,  10.7,   7.1,   6.4,   1.8, -0.9, 
		 -1.6,   1.9,   4.9,   2.0,  -2.0,   2.5,   2.5
	};

	return(lin_int(ff_ed-1,Hz-1,freqHz,NSHAW));
}
/*-------------------------------------------------------------*/
double ex_table(double frqHz)
{ 
/* Table of internal excitation levels at each Abs threshold in dB
vs frequency in Hz  - only for freqs up to 500 Hz
Above 500 Hz it's 3.73
	*/
  double lin_int(double *yy,double *xx, double x, unsigned long n);
  double temp;

#define NEX 35

  double Hz[NEX] = {
    0.,
			50.,     63.,    80.,   100.,    125.,   160.,  200.,   250.,
			315.,    400.,   500.,   630.,    750.,   800., 1000.,  1250.,
			1500.,   1600.,  2000.,  2500.,   3000.,  3150., 4000.,  5000.,
			6000.,   6300.,  8000.,  9000.,  10000., 11200.,12500., 14000.,
			17000.,  20000.
  };

// July29 2008
  double ex_at_thr[NEX] = {
//Hz	50.,     63.,    80.,   100.,    125.,   160.,  200.,   250.,
     32.0,
		 28.2,    24.0,   19.0,   15.68,   12.50,   9.90,  8.00,   6.30,
			5.3,     4.5,    3.73,   3.73,    3.73,   3.73,  3.73,   3.73,
			3.73,    3.73,   3.73,   3.73,    3.73,   3.73,  3.73,   3.73,
			3.73,    3.73,   3.73,   3.73,    3.73,   3.73,  3.73,   3.73,
			3.73,    3.73
  };

  temp = (lin_int(ex_at_thr-1,Hz-1,frqHz,NEX)) - 0.58;
  return (temp);
}
/*-------------------------------------------------------------*/

double alpha100(double G_dB)
{
  /* Table of alpha vs G_dB for equal N' at 100 dB */
  double lin_int(double *yy,double *xx, double x, unsigned long n);

#define NALPHA 16
	
  double Gtable[NALPHA] = {
        0.0,   -5.0,  -10.0,  -15.0,  -20.0,   -25.,  -30.0,  -35.0,
			-40.0,  -45.0,  -50.0,  -55.0,  -60.0,  -65.0,  -66.0,  -67.0
  };
	
  double alpha_table[NALPHA] = {
       0.2,  .21055, .22228, .23679, .25016, .26691, .28609, .30825,
		.33419,  .36499, .40229, .44869, .50953, .60486, .63930, .74660
  };
  return (lin_int(alpha_table-1,Gtable-1,G_dB,NALPHA));
}

/*-------------------------------------------------------------*/
double alpha90(double G_dB)  // not used for binloud
{
  /* Table of alpha vs G_dB for equal N' at 90 dB */
  double lin_int(double *yy,double *xx, double x, unsigned long n);
	
#define NALPHA 16
	
  double Gtable[NALPHA] = {
          0.0,   -5.0,  -10.0,  -15.0,  -20.0,  -25.0,   -30.0,  -35.0,
        -40.0,  -45.0,  -50.0,  -55.0,  -56.0,  -57.0,   -57.5,  -57.6
	};
 
  double alpha_table[NALPHA] = {
          0.2, .21182, .22512, .24021, .25749, .27747,  .30086, .32865,
       .36227, .40404, .45819, .53602, .55768, .58401,  .60021, .60383
	};
					
  return (lin_int(alpha_table-1,Gtable-1,G_dB,NALPHA));
}
/*-------------------------------------------------------------*/
double Btable(double alpha)    // Abs_const = 3.15 July 2008
{
#define NBETA 84
//   (10**(3.15/10))**alpha *[  3  **alpha - 2**alpha] =  0.1121805589689682

  double lin_int(double *yy,double *xx, double x, unsigned long n);
	
  long n_thr = 84;
	
  double alpha_tab[NBETA] = {
						 0.01,				 0.02,         0.03,				 0.04,				 0.05,				 0.06,
						 0.07,				 0.08,         0.09,				 0.10,			   0.11,				 0.12,
						 0.13,				 0.14,         0.15,				 0.16,			   0.17,				 0.18,
						 0.19,				 0.20,         0.21,				 0.22,			   0.23,				 0.24,
						 0.25,				 0.26,         0.27,				 0.28,			   0.29,				 0.30,
						 0.31,				 0.32,         0.33,				 0.34,			   0.35,				 0.36,
						 0.37,				 0.38,         0.39,				 0.40,			   0.41,				 0.42,
						 0.43,				 0.44,         0.45,				 0.46,			   0.47,				 0.48,
						 0.49,				 0.50,         0.51,				 0.52,			   0.53,				 0.54,
						 0.55,				 0.56,         0.57,				 0.58,			   0.59,				 0.60,
						 0.61,				 0.62,         0.63,				 0.64,			   0.65,				 0.66,
						 0.67,				 0.68,         0.69,				 0.70,			   0.71,				 0.72,
						 0.73,				 0.74,         0.75,				 0.76,			   0.77,				 0.78,
						 0.79,				 0.80,				 0.81,				 0.82,				 0.83,				 0.84,
  };
	
  double B[NBETA] = {
       0.00000745,   0.00286489,   0.02123525,   0.05963548,   0.11434001,   0.18140522, 
       0.25824680,   0.34337212,   0.43599798,   0.53579360,   0.64272733,   0.75697764, 
       0.87888188,   1.00890754,   1.14763730,   1.29576292,   1.45408529,   1.62351897, 
       1.80510049,   2.00000000,   2.20953560,   2.43519239,   2.67864332,   2.94177548, 
       3.22672057,   3.53589086,   3.87202177,   4.23822195,   4.63803274,   5.07549847, 
       5.55525029,   6.08260598,   6.66368953,   7.30557477,   8.01645839,   8.80586949, 
       9.68492403,  10.66663541,  11.76629495,   13.0019403,  14.39493503,  15.97068840, 
      17.75955552,  19.79796746,  22.12985902,   24.8084830,  27.89872998,  31.48011269, 
      35.65063332,  40.53182841,  46.27540062,   53.0720044,  61.16298125,  70.85617238, 
      82.54741976,  96.75008790, 114.13601383,  135.5929286, 162.30590590, 195.87430998, 
     238.48190977, 293.14777518, 364.10182148,  457.3558734, 581.58685724, 749.52775451, 
     980.20146588, 1302.5847854, 1761.7565165, 2429.4718719, 3422.8445880, 4938.3332941, 
     7315.5801445, 11161.610074, 17601.998271, 28811.025753, 49185.245609, 88083.090117, 
     166613.44715, 335629.95823, 727226.48839, 1715626.6308, 4473516.9907, 13137405.043
  };
  return (lin_int(B-1,alpha_tab-1,alpha,NBETA));
}
//-----------------------------------------------------------------------------------------
/* This function returns a linear version of K  */
double calc_K(double freqkHz)
{
  double lin_int(double *yy,double *xx, double x,unsigned long n);
	
  unsigned long n_thr = 23;
  double temp;
	
  double freq_kHz[23] = {
    0.0,
			0.05,  0.063, 0.08 ,   0.1, 0.125, 0.160,
			0.20,  0.25 , 0.315,   0.4, 0.5  , 0.630,  0.75,  0.8,
			1.00,  1.1  , 2.   ,   4.0, 8.0  ,10.000, 15.0 , 20.0
  };
  
  double KdB[23] = {
    20.0,
			13.5,   10.0,   7.2,  4.9 ,   3.1,   1.6,
			0.4,   -0.4,  -1.2, -1.85,  -2.4,  -2.7, -2.85, -2.9,
			-3.0,   -3.0, -3.0 , -3.0 ,  -3.0,  -3.0,  -3.0, -3.0
  };

  if(freqkHz >= 1.00)  return (pow(10.0,(-3.0/10.0)));
	
  temp = lin_int(KdB-1,freq_kHz-1,freqkHz,n_thr);
//  temp = lin_int(KdB,freq_kHz,freqkHz,n_thr);
  return (pow(10.0,(temp/10.0)));
}
/*--------------------------------------------------------------------*/
double so2phons(double sones) // Convert sones to phons
{
// put in sones (x), return phons(y)

#define NSONES 39

  double lin_int(double *yy,double *xx, double x, unsigned long n);
	
  long i;
  double lsones, log10_2, logsones[NSONES];
	

  // table of sones
  double sones_tab[NSONES] = {
        1.0e-15,    1.15e-9,   2.2e-9,    2.3e-8,
        4.15e-7,    7.15e-6,   0.000113,  0.00142,   0.00382,
        0.00761,    0.01018,   0.01868,   0.03132,   0.07349,
        0.14620,    0.25986,   0.42748,   0.66635,   1.00000,
        1.46046,    2.09208,   2.95659,   4.14132,   5.77265,
        8.03841,   11.23047,  15.84121,  22.72000,  32.88313,
       47.71964,   69.55788, 102.04293, 150.90542, 225.01522,
      337.64141,  507.82929, 760.53006,1104.78612,1430.60041
  };


  /*  matching table of phons */
  double stdphons[NSONES] = {
  		  -30.0,      -25.0,     -24.0,     -20.0,
	  	  -15.0,      -10.0,      -5.0,       0.0,       2.2,   
		  	  4.0,        5.0,       7.5,      10.0,      15.0,   
			   20.0,       25.0,      30.0,      35.0,      40.0,   
  			 45.0,       50.0,      55.0,      60.0,      65.0,   
	  		 70.0,       75.0,      80.0,      85.0,      90.0,   
		  	 95.0,      100.0,     105.0,     110.0,     115.0,   
        120.0,      125.0,     130.0,     135.0,     140.0
  };
	
  lsones = log10(sones)/log10(2.0);
  log10_2 = log10(2.0);
  for ( i = 0 ; i < NSONES ; i++) logsones[i] = log10(sones_tab[i])/log10_2;


  if(sones <= 1.15e-9) return(-25.0); // log10(sones = -25)/log10(2) 
	
	if(lsones > logsones[NSONES-1]+.0001) return(999.0); // about 140 dB !!
  return (lin_int(stdphons-1,logsones-1,lsones,NSONES));
}

/*-------------------------------------------------------------*/

/* known arrays xx[1..N] & yy[1..N], returns interpolated value of y for x
if x is out of range, returns yy[1] or yy[N] */

double lin_int(double *yy,double *xx, double x, unsigned long n)
{
	/*  Numerical Recipes  page 98 */
  void locate(double *xx, long n, double x, long *j);
  unsigned long j;
  
  locate(xx,n,x,&j);
  if(j < 1){
    return(yy[1]);
  }else if(j >= n){
    return(yy[n]);
  }else{
    return(yy[j]+(yy[j+1]-yy[j])*(x-xx[j])/(xx[j+1]-xx[j]));
  }
}
/*-------------------------------------------------------------*/

/* Given an array xx[1..n] and a value x, returns a value j such that x is
between xx[j] ans xx[j+1], j=0 or j=n indicates x is out of range */

void locate(double *xx, long n, double x, long *j) /*  Numerical Recipes  page 98 */
{
  long ascnd;
  long ju,jl,jm;
  
  jl = 0;
  ju = n + 1;
  ascnd=xx[n] > xx[1];
  while(ju-jl > 1) {
    jm = (ju + jl) >> 1;
    if((x > xx[jm]) == ascnd)
      jl = jm;
    else
      ju = jm;
  }
  *j = jl;
  
}
// Numerical Recipes 2nd ed  p 117    Copyright
/*-----------------------------------------------*/
#if 0
//   NOT USED
/* Function fqtoerb converts a frequency in Hz to the corresponding E value.*/
double fqtoerb (double fq)
{  return( C3 * log10((C2 * fq/1000.0) + 1.0)); }
#endif
/*-------------------------------------------------------------

* Function erbtofq converts the E value to its corresponding frequency in Hz.*/
double erbtofq(double E) /* (1000 * (10**(E/C3) -1) )/C2  */
{ return (1000.0 * (pow(10.0,(E/C3)) - 1.0) / C2); }
