// binloud.c   just one 361 point gaussian smoothing the specific loudness output
//             using dconvlv() from Numerical Recipes in C 2nd Edition
// Freq Limits of input 20 & 18000 Hz incl.

// This version compiled with Microsoft Visual C++ 6.0 to run under a Command Prompt (the old DOS)

#include "binloud.h"

#define GAUSS_PTS  361
#define CONVLV_PTS 512
#define CONVLV_PTS2 (2*CONVLV_PTS)

//global
short questions = TRUE, prt = TRUE;
char str[128];
int channel;

void main(int argc, char **argv)
{
  void dconvlv(double data[], unsigned long n, double respns[], unsigned long m,int isign, double ans[]);

	int isign = 1;
	long  i, c, ncomps=0, comp, n_corr;
	short freef = TRUE, d_corr = FALSE, first=TRUE, show_corr_file = FALSE;  // default is for free field frontal (freef = TRUE)
	short headphones = FALSE, h_corr=FALSE;
	short single_tone = FALSE, default_ohc = TRUE;
	int   nchans, Ecounter;
	double *compfq1,  *compfq2,  *compfq, *compint;
	double *compdB1,  *compdB2,  *compdB;
	double *sp_loud1, *sp_loud2, *inhib_sp_loud1, *inhib_sp_loud2;
	double *smooth1, *smooth2, *inhib1, *inhib2, *E_frqHz;
	double *gauss_wf, *gauss_wf_sav; // exponential function
	
	double E, Esig, Abs_const;
	double *erbdB, *corr_Hz, *corr_dB, corr_1k;
	double intensity, pl_l;
	double frqHz, frqkHz;
	double erb, p51, p, g, ELdB;
	double Broad1, abs_g, temp;


  double constant   = 0.06169745;

  double inhib_P = 1.5979;   // inhib_P adjusted to give 1 sone = 40 dB and binaural sones = monaural * 1.5
	char	corr_file[80];
	FILE *corr_fd;
	
	// loudness
	double mono_sones, inhib_sones1, inhib_sones2, bin_sones;
	double mono_sones1, mono_sones2;
	double sp_loud, bin_phons, alpha, Ethrq_Norm_dB;
	double G_exNorm, G_dBNorm, AAA, BBB, EthrqG;
	double EthrQ;
	
	extern char *optarg;
	extern int optind;
	
	compfq1	  			  = dvector(1l,(long)N_COMP);
	compdB1 					= dvector(1l,(long)N_COMP);
	compfq2	  			  = dvector(1l,(long)N_COMP);
	compdB2 					= dvector(1l,(long)N_COMP);
	compint					  = dvector(1l,(long)N_COMP);
	erbdB							= dvector(1l,(long)N_COMP);
	
	corr_Hz						= dvector(1l,(long)MAX_THR);
	corr_dB						= dvector(1l,(long)MAX_THR);
	
	sp_loud1          = dvector(1l,(long)CONVLV_PTS);   // original mono specific loud, channel 1
	sp_loud2          = dvector(1l,(long)CONVLV_PTS);   // channel 2  1st N_ERBS (372) points sp_loud, rest up to CONVLV_PTS (512) zeros
	inhib_sp_loud1    = dvector(1l,(long)N_ERBS);       // inhibited mono specific loud, channel 1
	inhib_sp_loud2    = dvector(1l,(long)N_ERBS);       // inhibited mono specific loud, channel 2
	smooth1           = dvector(1l,(long)CONVLV_PTS2);  // intermediate smoothed specific loud, channel 1
	smooth2           = dvector(1l,(long)CONVLV_PTS2);  // intermediate smoothed specific loud, channel 2
	inhib1            = dvector(1l,(long)N_ERBS);       // intermediate inhibition or gain values used to divide sp_loud1
	inhib2            = dvector(1l,(long)N_ERBS);       // intermediate inhibition or gain values used to divide sp_loud2
	E_frqHz           = dvector(1l,(long)N_ERBS);
	gauss_wf          = dvector(1l,(long)CONVLV_PTS);   // 1st GAUSS_PTS (361) of 512  set to wrapped round gaussian 
	gauss_wf_sav      = dvector(1l,(long)CONVLV_PTS);   // save copy of gauss function
  
  for ( i = 1 ; i <= CONVLV_PTS ; i++) gauss_wf[i] = sp_loud1[i] = sp_loud2[i]= 0.0;

	while ((c = getopt(argc, argv, "Ipqhc:ds")) != EOF)
		switch (c) {
		case 'I': info(); exit(0);
		case 'p': prt = FALSE; break;
		case 'q': questions = FALSE; break;
		case 'h': headphones=TRUE; freef = FALSE; break;
		case 'c': strcpy(corr_file,optarg); headphones = TRUE; h_corr = TRUE; freef = FALSE; break;
		case 'd': d_corr = TRUE; break;   // freef left true
    case 's': show_corr_file = TRUE; break;
		case '?': printf("unknown file option %s\n",argv[optind]); exit(1);
	}
	
	if(prt){
		//    printf("\n\n---------------------------------------------------------------------------\n\n");
		if(freef && !d_corr)  printf("\nFree-field frontal: Shaw & middle ear correction used\n");
		if(freef && d_corr) printf("\nFree-field diffuse: Free-diffuse & middle ear correction used\n");
		if(headphones && !h_corr) printf("\nHeadphones (flat reponse at eardrum) used\n");
		if(headphones && h_corr)  printf("\nHeadphones with correction file %s used\n",corr_file);
	}
	
	if(h_corr && headphones){// read corrections into corr_Hz and corr_dB, number of values into n_corr
		if((corr_fd = fopen(corr_file,"r")) == NULL){
			printf("can't open %s\n",corr_file); exit(1);
		}
		read_corr(&n_corr,corr_Hz,corr_dB,corr_fd);
		corr_1k = lin_int(corr_dB,corr_Hz,1000.0,n_corr);
	} else if(!h_corr) corr_1k = 0.0;
	
  if(show_corr_file){
    printf("  Freq, Hz     Corr dB\n");
    printf("           n = %d\n",n_corr);
    for( i = 1 ; i <= n_corr ; i++) printf("%2d: %7.1f     %7.1f\n",i,corr_Hz[i], corr_dB[i]);
    printf("\n\n");
    exit(0);
  }

label_input:
	nchans  = 2; // only for dichotic i.e. sounds to both ears
// channel = 1 for 1st channel,  channel = 2 for 2nd channel
	channel = 0;

	do{ // while channel < nchans
		channel++; // only for dichotic
		if(channel == 1){
			compfq = compfq1;   // use compfq to point to compfq1 for channel 1
			compdB = compdB1;   // use compdB to point to compdB1 for channel 1
		}else if(channel == 2){
			compfq = compfq2;   // use compfq to point to compfq2 for channel 2
			compdB = compdB2;   // use compdB to point to compdB2 for channel 2
		}
		
		if(questions) {
			fprintf(stderr,"\ninput for channel %1d ........",channel);
			if(channel == 1 && !first) fprintf(stderr,"or exit");
			fprintf(stderr,"\n");
		}
		first = FALSE;
		
		input(compfq,compdB,&ncomps);
		
		if(ncomps == 0) silent_channel(compfq,compdB,&ncomps);

		if(prt){
			printf("\nchannel %1d freq/level input:\n",channel);
			for(i = 1 ; i <= ncomps ; i++ ) printf("%6.1f Hz    %5.2f dB\n",compfq[i],compdB[i]);
			printf("------\n");
		}
		
		if(freef){
			if(!d_corr){
				for ( i = 1 ; i <= ncomps ; i++ )
					compdB[i] = compdB[i] + Shaw_cor(compfq[i]) - mid_corr(compfq[i]);
			}	else if(d_corr){
				for ( i = 1 ; i <= ncomps ; i++ )
					compdB[i] = compdB[i] + diff_cor(compfq[i]) - mid_corr(compfq[i]);
			}
		}else if(!freef){  // headphone response
			for ( i = 1 ; i <= ncomps ; i++ )
				compdB[i] = compdB[i] - mid_corr(compfq[i]);
			if(h_corr)
				for ( i = 1 ; i <= ncomps ; i++ )
					compdB[i] = compdB[i] + thr_corr(compfq[i],n_corr,corr_Hz,corr_dB);
		} // if(freef)
		
		for ( i = 1 ; i <= ncomps ; i++ )
			compint[i] = pow(10.0,compdB[i]/10.0);
		
		
		//-----------------------------------------------------------------
		
		for ( i = 1 ; i <= ncomps ; i++ ){
			intensity = 0.0;
			frqHz = compfq[i];
			frqkHz = frqHz/1000.0;
			erb = C1 * (C2 * frqkHz + 1.0);
			p = 4.0 * frqHz/erb;
			for (comp = 1 ; comp <= ncomps ; comp++ ){
				g = fabs((compfq[comp] - frqHz)/frqHz);
				if(g > 4.0) break;
				intensity = intensity + (1.0 + p*g) * exp(-p * g) * compint[comp];
			}
			erbdB[i] = 10.0 * log10(intensity);
		}
		
//-----------------------------------------------------------------
		
		
		mono_sones = 0.0;
		
		Ecounter = 0; // for keeping track of howmany Es we've gone through
		for( E = ESTART ; E <= EEND+ESTEP/2.0 ; E += ESTEP){
			Ecounter++;  // goes from 1 to 372
			frqHz = erbtofq(E);
			E_frqHz[Ecounter] = frqHz; // save frequencies of erb scale in Hz
			frqkHz = frqHz/1000.0;
			
			erb = C1 * (C2 * frqkHz + 1.0);
			p51 = 4.0 * frqHz/erb;
			Esig = 0.0;
			for ( comp = 1 ; comp <= ncomps ; comp++ ){
				g = (compfq[comp]-frqHz)/frqHz;
				// To prevent floating point errors in line 777 (when taking exp(-p*g))
				// g is not allowed to be greater than 4.
				if(g>4.0) break;
				pl_l = 0.35;
				if(g < 0.0) p= p51 - pl_l*(p51/p51_1k)*(erbdB[comp]-51.0);
				// don't let pu change with level
				if(g >= 0.0) p = p51;
				
				// p lower can go -ve for very intense (>130dB tones input)
				// put in a fix, this only affects stuff above about 120 dB !!
				if(p < 0.1) p = 0.1;
				g = fabs(g);
				Esig = Esig+(1.0+p*g)*exp(-p*g)*compint[comp];
			}
			
			if (Esig < 1.0e-10) Esig = 1.0e-10; 
			// minimum of -100 dB
			ELdB = 10.0 * log10(Esig);
			
//*********************************
		
// work out specific loudness for excitation at this frequency
// EthrqG is a constant, the same as EthrQ * G
			Abs_const = ex_table(1000.0);
			EthrqG = pow(10.0,(Abs_const/10.0));
			sp_loud = 0.0;
			G_dBNorm = 0.0;
			
			if(frqHz > 500.0){
				Ethrq_Norm_dB = Abs_const;
				G_exNorm = 1.0;
				G_dBNorm = 0.0;
				alpha = 0.2;
				BBB = 2.0;
			}else if(frqHz <= 500.0){
				Ethrq_Norm_dB = ex_table(frqHz);
				G_exNorm = 1.0/pow(10.0,((Ethrq_Norm_dB - Abs_const)/10.0));
				G_dBNorm = 10.0 * log10(G_exNorm);
				alpha = alpha100(G_dBNorm);    // alpha100

        BBB = Btable(alpha);
      }
			
			EthrQ = pow(10.0,(Ethrq_Norm_dB/10.0));
			
			AAA = BBB*EthrqG;
			
			if(ELdB > Ethrq_Norm_dB-30.){
				if(Esig <= 1.0e10)
					sp_loud = (constant*  ( pow((Esig * G_exNorm + AAA),alpha) - pow(AAA,alpha) ));
				else 
					sp_loud = constant *pow((Esig/1.0707),0.2);
			}
            
			
			if(Esig < EthrQ){
				sp_loud = sp_loud * pow(((2.0 * Esig)/(Esig + EthrQ)),1.5); //! eq(8) p 228
			}
			
			if(channel == 1) sp_loud1[Ecounter]= sp_loud*ESTEP;  // saving all specific loudnesses for channel 1
			if(channel == 2) sp_loud2[Ecounter]= sp_loud*ESTEP;
			
			
			
//    Uncomment if you want these values printed out
//      if(prt){  // print specific loudness in ESTEP ERB (0.1 for this program)
//        if(sp_loud*ESTEP > 0.00001){
//          if(channel == 1) printf(" %8.4f%11.2f     %11.2f   %15.5f\n",frqkHz,Ethrq_Norm_dB,ELdB,sp_loud1[Ecounter]);
//          if(channel == 2) printf(" %8.4f%11.2f     %11.2f   %15.5f\n",frqkHz,Ethrq_Norm_dB,ELdB,sp_loud2[Ecounter]);
//        }
//			}
			
			mono_sones = mono_sones + sp_loud;     // sp_loud has NOT been multiplied by ESTEP
		}
		
		mono_sones  = mono_sones  * ESTEP;       
		if(channel == 1) mono_sones1 = mono_sones;
		if(channel == 2) mono_sones2 = mono_sones; 
	}while(channel < nchans);
  
  
  
  // sp_loud1[] & sp_loud2[] hold the original specific loudness values
  // that will be inhibited by inhib1[] & inhib2[] calculated later
  for ( i = 1 ; i <= N_ERBS ; i++){
	  if(sp_loud1[i] < 1.0e-13) sp_loud1[i] = 1.0e-13; // to stop problems when dividing in inhib_sp_loud1[i] calc
	  if(sp_loud2[i] < 1.0e-13) sp_loud2[i] = 1.0e-13;
  }
  
//  if(prt)printf("\nSpecific loudness for Chan1 & Chan2\n\n");
  
  for( i = 1 ; i <= CONVLV_PTS2 ; i++ ) smooth1[i] = smooth2[i] = 0.0;
  
  //------------------------------------------------------------------------------------------
// Calc gauss filter in 'wrap around' order (Num Rec 2nd ed Ch 13 page 543 for dconvlv ( the d is for double precision)
  Broad1 = 0.08;        // lower Broad1, broader filter
  
  for ( i = 1 ; i <= GAUSS_PTS/2 +1; i++ ){  // i = 1 to 181
      g = (float)(i - 1)*0.1;
      abs_g = fabs(g);
      temp = Broad1 * abs_g;
      gauss_wf[i] = exp(-temp*temp);
  }
  for ( i = 1 ; i <= GAUSS_PTS/2 ; i++ ){  // i = 182 to 361  2 -> 361   180 -> 182
      g = (float)(i)*0.1;
      abs_g = fabs(g);
      temp = Broad1 * abs_g;
      gauss_wf[362 - i] = exp(-temp*temp);
  }
  for ( i = 1 ; i <= CONVLV_PTS; i++ ) gauss_wf_sav[i] = gauss_wf[i];

//---------------------------------------------------------------------------
  // CHECK SMOOTHING WITH ONE SPIKE IN SPECIFIC LOUDNESS ARRAY
/*  for ( i = 1 ; i <= N_ERBS ; i++) sp_loud1[i] = sp_loud2[i] = 0.0;  // set sp_loud to zeros
  sp_loud1[182] = 1.0;
  sp_loud2[182] = 1.0;
//---------------------------------------------------------------------------
*/  
  
// Feb 29th 2008   changed smoothing code to use Num Rec convolution dconvlv.c (version for doubles
  
// Smear the specific loudness arrays with a gaussian(Broad) equation  

  dconvlv(sp_loud1,CONVLV_PTS,gauss_wf,GAUSS_PTS,isign,smooth1);
  for ( i = 1 ; i <= CONVLV_PTS; i++ ) gauss_wf[i] = gauss_wf_sav[i];  // restore gauss_wf[]
  dconvlv(sp_loud2,CONVLV_PTS,gauss_wf,GAUSS_PTS,isign,smooth2);


// The specific loudness data is in sp_loud1[i] & sp_loud2[i] i = 1...N_ERBS (1 to 372 points)
// for convlv( ) it needs to be 2^n long so copy into 512 point float array
// the gauss_wt is 361 points long,

 
// calc inhib1[] & inhib2[]
  for ( i = 1 ; i <= N_ERBS; i++ ){
	  inhib1[i] = 2.0/(1.0+pow(1.0/cosh(smooth2[i]/smooth1[i]),inhib_P));
	  inhib2[i] = 2.0/(1.0+pow(1.0/cosh(smooth1[i]/smooth2[i]),inhib_P));
  }
  
  inhib_sones1 = inhib_sones2 = 0.0;
	Ecounter = 0; // for keeping track of howmany Es we've gone through
  for ( i = 1 ; i <= N_ERBS; i++ ){
	  inhib_sp_loud1[i] = sp_loud1[i]/inhib1[i];
	  inhib_sones1 += inhib_sp_loud1[i];
	  inhib_sp_loud2[i] = sp_loud2[i]/inhib2[i];
	  inhib_sones2 += inhib_sp_loud2[i];

		Ecounter++;  // goes from 1 to 372
  }
  
  bin_sones = inhib_sones1 +  inhib_sones2;
  bin_phons = so2phons(bin_sones);

  printf("loudness       =%10.3f sones\n",bin_sones);
  printf("loudness level =%8.1f   phons\n",bin_phons);
  printf("------------------------------------\n");
  fflush(stdout);
  goto label_input;
}

//--------------------------------------------------------------------------
void info(void)
{
	printf("\nversion 2, uses modified loudness model G&M (2006) JASA 120 585-588\n\n");
	printf("Only for binaural stimuli\n\n\n");
	printf("binloud -p -q -d -h -c corr_file -I \n\n");
	printf(" -p turns off some printing\n");
	printf(" -q turns off questions\n");
	printf(" -d for diffuse field presentation   (free field frontal default)\n");
	printf(" -h for headphone presentation with no correction\n");
	printf(" -c 'correction_file' for headphone presentation with a headphone correction file\n");
	printf(" -I for this information\n");
}
/*--------------------------------------------------------*/
